window.Gleam.initSearch([{"doc":"gleastsq","title":"gleastsq","content":"# gleastsq\n\n[![Package Version](https://img.shields.io/hexpm/v/gleastsq)](https://hex.pm/packages/gleastsq)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/gleastsq/)\n\nA least squares curve fitting library for Gleam. This library uses the [Nx](https://hexdocs.pm/nx/Nx.html)\nlibrary from Elixir under the hood to perform matrix operations.\n\n## Which method should I use?\n\nThe library provides four functions for curve fitting: `least_squares`, `gauss_newton`, `levenberg_marquardt` and `trust_region_reflective`.\n\n### Least Squares\n\nThe `least_squares` function is just an alias for the `levenberg_marquardt` function.\n\n## Levenberg-Marquardt\n\nIdeal for non-linear least squares problems, particularly when the initial guess is far from the solution. It combines the benefits of the Gauss-Newton method and gradient descent, making it robust and efficient for various scenarios. However, it requires careful tuning of the damping parameter to balance convergence speed and stability.\n\n## Trust-Region Reflective\n\nBest suited for large-scale problems or those with constraints, this method ensures that each iteration stays within a predefined &quot;trust region,&quot; preventing large, unstable steps. It is reliable and effective for challenging optimization problems but can be computationally intensive.\n\n* Constrained parameters is not supported yet.\n\n## Gauss-Newton\n\nEfficient for problems where residuals are small and the initial guess is close to the true solution. It approximates the Hessian matrix, leading to faster convergence for well-behaved problems. However, it may struggle with highly non-linear problems or poor initial guesses, as it lacks the robustness of the Levenberg-Marquardt and trust-region reflective methods.\n\n## Installation\n\n```sh\ngleam add gleastsq\n```\n\n```gleam\nimport gleam/io\nimport gleastsq\n\nfn parabola(x: Float, params: List(Float)) -&gt; Float {\n  let assert [a, b, c] = params\n  a *. x *. x +. b *. x +. c\n}\n\npub fn main() {\n  let x = [0.0, 1.0, 2.0, 3.0, 4.0, 5.0]\n  let y = [0.0, 1.0, 4.0, 9.0, 16.0, 25.0]\n  let initial_guess = [1.0, 1.0, 1.0]\n\n  let assert Ok(result) =\n    gleastsq.least_squares(x, y, parabola, initial_guess, opts: [])\n\n  io.debug(result) // [1.0, 0.0, 0.0] (within numerical error)\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/gleastsq&gt;.\n","url":"index.html"},{"doc":"gleastsq/errors","title":"FitErrors","content":"pub type FitErrors {\n  NonConverged\n  WrongParameters(String)\n  JacobianTaskError\n}\n\nNonConverged\n\n\nWrongParameters(String)\n\n\nJacobianTaskError\n\n\nSynonyms:\ngleastsq/errors.FitErrors\ngleastsq/errors FitErrors","url":"gleastsq/errors.html#FitErrors"},{"doc":"gleastsq/errors","title":"gleastsq/errors","content":"","url":"gleastsq/errors.html"},{"doc":"gleastsq/options","title":"LeastSquareOptions","content":"pub type LeastSquareOptions {\n  Iterations(Int)\n  Epsilon(Float)\n  Tolerance(Float)\n  Damping(Float)\n  DampingIncrease(Float)\n  DampingDecrease(Float)\n}\n\nIterations(Int)\n\n\nEpsilon(Float)\n\n\nTolerance(Float)\n\n\nDamping(Float)\n\n\nDampingIncrease(Float)\n\n\nDampingDecrease(Float)\n\n\nSynonyms:\ngleastsq/options.LeastSquareOptions\ngleastsq/options LeastSquareOptions","url":"gleastsq/options.html#LeastSquareOptions"},{"doc":"gleastsq/options","title":"gleastsq/options","content":"","url":"gleastsq/options.html"},{"doc":"gleastsq","title":"gauss_newton","content":"pub fn gauss_newton(\n  x: List(Float),\n  y: List(Float),\n  func: fn(Float, List(Float)) -&gt; Float,\n  initial_params: List(Float),\n  opts opts: List(LeastSquareOptions),\n) -&gt; Result(List(Float), FitErrors)\n The `gauss_newton` function performs a basic least squares optimization algorithm.\n It is used to find the best-fit parameters for a given model function to a set of data points.\n This function takes as input the data points, the model function, and several optional parameters to control the optimization process.\n\n # Parameters\n - `x` (List(Float))\n     A list of x-values of the data points.\n - `y` (List(Float))\n     A list of y-values of the data points.\n - `func` (fn(Float, List(Float)) -&gt; Float)\n     The model function that takes an x-value and a list of parameters, and returns the corresponding y-value.\n - `initial_params` (List(Float))\n     A list of initial guesses for the parameters of the model function.\n - `opts` (List(LeastSquareOptions))\n     A list of optional parameters to control the optimization process.\n     The available options are:\n     - `Iterations(Int)`: The maximum number of iterations to perform. Default is 100.\n     - `Epsilon(Float)`: A small value to change x when calculating the derivatives for the function. Default is 0.0001.\n     - `Tolerance(Float)`: The convergence tolerance. Default is 0.0001.\n     - `Damping(Float)`: The value of the damping parameter. Default is 0.001.\n\n # Example\n \n import gleam/io\n import gleastsq\n import gleastsq/options.{Iterations, Tolerance}\n\n fn parabola(x: Float, params: List(Float)) -&gt; Float {\n   let assert [a, b, c] = params\n   a *. x *. x +. b *. x +. c\n }\n\n pub fn main() {\n   let x = [0.0, 1.0, 2.0, 3.0, 4.0, 5.0]\n   let y = [0.0, 1.0, 4.0, 9.0, 16.0, 25.0]\n   let initial_guess = [1.0, 1.0, 1.0]\n\n   let assert Ok(result) =\n     gleastsq.gauss_newton(\n       x,\n       y,\n       parabola,\n       initial_guess,\n       opts: [Iterations(1000), Tolerance(0.001)]\n     )\n\n   io.debug(result) // [1.0, 0.0, 0.0] (within numerical error)\n }\n \nSynonyms:\ngleastsq.gauss_newton\ngleastsq gauss_newton","url":"gleastsq.html#gauss_newton"},{"doc":"gleastsq","title":"least_squares","content":"pub fn least_squares(\n  x: List(Float),\n  y: List(Float),\n  func: fn(Float, List(Float)) -&gt; Float,\n  initial_params: List(Float),\n  opts opts: List(LeastSquareOptions),\n) -&gt; Result(List(Float), FitErrors)\n The `least_squares` function is an alias for the `levenberg_marquardt` function.\n Check the documentation of the `levenberg_marquardt` function for more information.\nSynonyms:\ngleastsq.least_squares\ngleastsq least_squares","url":"gleastsq.html#least_squares"},{"doc":"gleastsq","title":"levenberg_marquardt","content":"pub fn levenberg_marquardt(\n  x: List(Float),\n  y: List(Float),\n  func: fn(Float, List(Float)) -&gt; Float,\n  initial_params: List(Float),\n  opts opts: List(LeastSquareOptions),\n) -&gt; Result(List(Float), FitErrors)\n The `levenberg_marquardt` function performs the Levenberg-Marquardt optimization algorithm.\n It is used to solve non-linear least squares problems. This function takes as input the data points,\n the model function, and several optional parameters to control the optimization process.\n\n # Parameters\n - `x` (List(Float))\n     A list of x-values of the data points.\n - `y` (List(Float))\n     A list of y-values of the data points.\n - `func` (fn(Float, List(Float)) -&gt; Float)\n     The model function that takes an x-value and a list of parameters, and returns the corresponding y-value.\n - `initial_params` (List(Float))\n     A list of initial guesses for the parameters of the model function.\n - `opts` (List(LeastSquareOptions))\n     A list of optional parameters to control the optimization process.\n     The available options are:\n     - `Iterations(Int)`: The maximum number of iterations to perform. Default is 100.\n     - `Epsilon(Float)`: A small value to change x when calculating the derivatives for the function. Default is 0.0001.\n     - `Tolerance(Float)`: The convergence tolerance. Default is 0.0001.\n     - `Damping(Float)`: The initial value of the damping parameter. Default is 0.0001.\n     - `DampingIncrease(Float)`: The factor by which the damping parameter is increased when a step fails. Default is 10.0.\n     - `DampingDecrease(Float)`: The factor by which the damping parameter is decreased when a step succeeds. Default is 0.1.\n\n # Example\n \n import gleam/io\n import gleastsq\n import gleastsq/options.{Iterations, Tolerance}\n\n fn parabola(x: Float, params: List(Float)) -&gt; Float {\n   let assert [a, b, c] = params\n   a *. x *. x +. b *. x +. c\n }\n\n pub fn main() {\n   let x = [0.0, 1.0, 2.0, 3.0, 4.0, 5.0]\n   let y = [0.0, 1.0, 4.0, 9.0, 16.0, 25.0]\n   let initial_guess = [1.0, 1.0, 1.0]\n\n   let assert Ok(result) =\n     gleastsq.levenberg_marquardt(\n       x,\n       y,\n       parabola,\n       initial_guess,\n       opts: [Iterations(1000), Tolerance(0.001)]\n     )\n\n   io.debug(result) // [1.0, 0.0, 0.0] (within numerical error)\n }\n \nSynonyms:\ngleastsq.levenberg_marquardt\ngleastsq levenberg_marquardt","url":"gleastsq.html#levenberg_marquardt"},{"doc":"gleastsq","title":"trust_region_reflective","content":"pub fn trust_region_reflective(\n  x: List(Float),\n  y: List(Float),\n  func: fn(Float, List(Float)) -&gt; Float,\n  initial_params: List(Float),\n  opts opts: List(LeastSquareOptions),\n) -&gt; Result(List(Float), FitErrors)\n The `trust_region_reflective` function performs a least squares optimization using the Trust Region Reflective algorithm.\n It is used to find the best-fit parameters for a given model function to a set of data points.\n This function takes as input the data points, the model function, and several optional parameters to control the optimization process.\n\n # Parameters\n - `x` (List(Float))\n     A list of x-values of the data points.\n - `y` (List(Float))\n     A list of y-values of the data points.\n - `func` (fn(Float, List(Float)) -&gt; Float)\n     The model function that takes an x-value and a list of parameters, and returns the corresponding y-value.\n - `initial_params` (List(Float))\n     A list of initial guesses for the parameters of the model function.\n - `opts` (List(LeastSquareOptions))\n     A list of optional parameters to control the optimization process.\n     The available options are:\n     - `Iterations(Int)`: The maximum number of iterations to perform. Default is 100.\n     - `Epsilon(Float)`: A small value to change x when calculating the derivatives for the function. Default is 0.0001.\n     - `Tolerance(Float)`: The convergence tolerance. Default is 0.00001.\n     - `Damping(Float)`: The value of the damping parameter. Default is 0.001.\n\n # Example\n \n import gleam/io\n import gleastsq\n import gleastsq/options.{Iterations, Tolerance}\n\n fn parabola(x: Float, params: List(Float)) -&gt; Float {\n   let assert [a, b, c] = params\n   a *. x *. x +. b *. x +. c\n }\n\n pub fn main() {\n   let x = [0.0, 1.0, 2.0, 3.0, 4.0, 5.0]\n   let y = [0.0, 1.0, 4.0, 9.0, 16.0, 25.0]\n   let initial_guess = [1.0, 1.0, 1.0]\n\n   let assert Ok(result) =\n     gleastsq.trust_region_reflective(\n       x,\n       y,\n       parabola,\n       initial_guess,\n       opts: [Iterations(1000), Tolerance(0.001)]\n     )\n\n   io.debug(result) // [1.0, 0.0, 0.0] (within numerical error)\n }\n \nSynonyms:\ngleastsq.trust_region_reflective\ngleastsq trust_region_reflective","url":"gleastsq.html#trust_region_reflective"},{"doc":"gleastsq","title":"gleastsq","content":"","url":"gleastsq.html"}]);